@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # This file is part of Glances.
 #
@@ -13,12 +12,12 @@ import os
 import re
 import threading
 
-from glances.globals import listkeys, iteritems, amps_path
+from glances.globals import amps_path, listkeys
 from glances.logger import logger
 from glances.processes import glances_processes
 
 
-class AmpsList(object):
+class AmpsList:
     """This class describes the optional application monitoring process list.
 
     The AMP list is a list of processes with a specific monitoring action.
@@ -57,9 +56,9 @@ class AmpsList(object):
                 try:
                     amp = __import__(os.path.basename(amp_module))
                 except ImportError as e:
-                    logger.warning("Missing Python Lib ({}), cannot load AMP {}".format(e, amp_name))
+                    logger.warning(f"Missing Python Lib ({e}), cannot load AMP {amp_name}")
                 except Exception as e:
-                    logger.warning("Cannot load AMP {} ({})".format(amp_name, e))
+                    logger.warning(f"Cannot load AMP {amp_name} ({e})")
                 else:
                     # Add the AMP to the dictionary
                     # The key is the AMP name
@@ -69,7 +68,7 @@ class AmpsList(object):
                     # Load the AMP configuration
                     self.__amps_dict[amp_name].load_config(self.config)
         # Log AMPs list
-        logger.debug("AMPs list: {}".format(self.getList()))
+        logger.debug(f"AMPs list: {self.getList()}")
 
         return True
 
@@ -91,7 +90,7 @@ class AmpsList(object):
         processlist = glances_processes.get_list()
 
         # Iter upon the AMPs dict
-        for k, v in iteritems(self.get()):
+        for k, v in self.get().items():
             if not v.enable():
                 # Do not update if the enable tag is set
                 continue
@@ -106,9 +105,9 @@ class AmpsList(object):
 
             amps_list = self._build_amps_list(v, processlist)
 
-            if len(amps_list) > 0:
+            if amps_list:
                 # At least one process is matching the regex
-                logger.debug("AMPS: {} processes {} detected ({})".format(len(amps_list), k, amps_list))
+                logger.debug(f"AMPS: {len(amps_list)} processes {k} detected ({amps_list})")
                 # Call the AMP update method
                 thread = threading.Thread(target=v.update_wrapper, args=[amps_list])
                 thread.start()
@@ -126,21 +125,17 @@ class AmpsList(object):
 
         Search application monitored processes by a regular expression
         """
-        ret = []
         try:
             # Search in both cmdline and name (for kernel thread, see #1261)
-            for p in processlist:
-                if (re.search(amp_value.regex(), p['name']) is not None) or (
-                    p['cmdline'] is not None
-                    and p['cmdline'] != []
-                    and re.search(amp_value.regex(), ' '.join(p['cmdline'])) is not None
-                ):
-                    ret.append(
-                        {'pid': p['pid'], 'cpu_percent': p['cpu_percent'], 'memory_percent': p['memory_percent']}
-                    )
+            ret = [
+                {'pid': p['pid'], 'cpu_percent': p['cpu_percent'], 'memory_percent': p['memory_percent']}
+                for p in processlist
+                if re.search(amp_value.regex(), p['name'])
+                or ((cmdline := p.get('cmdline')) and re.search(amp_value.regex(), ' '.join(cmdline)))
+            ]
 
         except (TypeError, KeyError) as e:
-            logger.debug("Can not build AMPS list ({})".format(e))
+            logger.debug(f"Can not build AMPS list ({e})")
 
         return ret
 
