@@ -29,7 +29,7 @@ from configparser import ConfigParser, NoOptionError, NoSectionError
 from datetime import datetime
 from operator import itemgetter, methodcaller
 from statistics import mean
-from typing import Any, Union
+from typing import Any, Optional, Union
 from urllib.error import HTTPError, URLError
 from urllib.parse import urlparse
 from urllib.request import Request, urlopen
@@ -393,13 +393,22 @@ def dictlist(data, item):
         return None
 
 
-def json_dumps_dictlist(data, item):
+def dictlist_json_dumps(data, item):
     dl = dictlist(data, item)
     if dl is None:
         return None
     return json_dumps(dl)
 
 
+def dictlist_first_key_value(data: list[dict], key, value) -> Optional[dict]:
+    """In a list of dict, return first item where key=value or none if not found."""
+    try:
+        ret = next(item for item in data if key in item and item[key] == value)
+    except StopIteration:
+        ret = None
+    return ret
+
+
 def string_value_to_float(s):
     """Convert a string with a value and an unit to a float.
     Example:
@@ -467,18 +476,32 @@ def folder_size(path, errno=0):
         return ret_size, ret_err
 
 
-def weak_lru_cache(maxsize=128, typed=False):
+def _get_ttl_hash(ttl):
+    """A simple (dummy) function to return a hash based on the current second.
+    TODO: Implement a real TTL mechanism.
+    """
+    if ttl is None:
+        return 0
+    now = datetime.now()
+    return now.second
+
+
+def weak_lru_cache(maxsize=1, typed=False, ttl=None):
     """LRU Cache decorator that keeps a weak reference to self
+
+    Warning: When used in a class, the class should implement __eq__(self, other) and __hash__(self) methods
+
     Source: https://stackoverflow.com/a/55990799"""
 
     def wrapper(func):
         @functools.lru_cache(maxsize, typed)
-        def _func(_self, *args, **kwargs):
+        def _func(_self, *args, ttl_hash=None, **kwargs):
+            del ttl_hash  # Unused parameter, but kept for compatibility
             return func(_self(), *args, **kwargs)
 
         @functools.wraps(func)
         def inner(self, *args, **kwargs):
-            return _func(weakref.ref(self), *args, **kwargs)
+            return _func(weakref.ref(self), *args, ttl_hash=_get_ttl_hash(ttl), **kwargs)
 
         return inner
 
@@ -487,7 +510,7 @@ def weak_lru_cache(maxsize=128, typed=False):
 
 def namedtuple_to_dict(data):
     """Convert a namedtuple to a dict, using the _asdict() method embedded in PsUtil stats."""
-    return {k: (v._asdict() if hasattr(v, '_asdict') else v) for k, v in data.items()}
+    return {k: (v._asdict() if hasattr(v, '_asdict') else v) for k, v in list(data.items())}
 
 
 def list_of_namedtuple_to_list_of_dict(data):
@@ -499,3 +522,12 @@ def replace_special_chars(input_string, by=' '):
     """Replace some special char by another in the input_string
     Return: the string with the chars replaced"""
     return input_string.replace('\r\n', by).replace('\n', by).replace('\t', by)
+
+
+def atoi(text):
+    return int(text) if text.isdigit() else text
+
+
+def natural_keys(text):
+    """Return a text in a natural/human readable format."""
+    return [atoi(c) for c in re.split(r'(\d+)', text)]
