@@ -8,14 +8,16 @@
 
 """Curses interface class."""
 
+import functools
 import getpass
 import sys
 
 from glances.events_list import glances_events
-from glances.globals import MACOS, WINDOWS, disable, enable, itervalues, nativestr, u
+from glances.globals import MACOS, WINDOWS, disable, enable, nativestr, u
 from glances.logger import logger
+from glances.outputs.glances_colors import GlancesColors
 from glances.outputs.glances_unicode import unicode_message
-from glances.processes import glances_processes, sort_processes_key_list
+from glances.processes import glances_processes, sort_processes_stats_list
 from glances.timer import Timer
 
 # Import curses library for "normal" operating system
@@ -83,6 +85,7 @@ class _GlancesCurses:
         'T': {'switch': 'network_sum'},
         'u': {'sort_key': 'username'},
         'U': {'switch': 'network_cumul'},
+        'V': {'switch': 'disable_vms'},
         'w': {'handler': '_handle_clean_logs'},
         'W': {'switch': 'disable_wifi'},
         'x': {'handler': '_handle_clean_critical_logs'},
@@ -95,16 +98,16 @@ class _GlancesCurses:
         # 'DOWN' > Down in the server list
     }
 
-    _sort_loop = sort_processes_key_list
+    _sort_loop = sort_processes_stats_list
 
     # Define top menu
     _top = ['quicklook', 'cpu', 'percpu', 'gpu', 'mem', 'memswap', 'load']
     _quicklook_max_width = 58
 
     # Define left sidebar
-    # This default list is also defined in the glances/outputs/static/js/uiconfig.json
-    # file for the web interface
-    # Both can be overwritten by the configuration file ([outputs] left_menu option)
+    # This variable is used in the make webui task in order to generate the
+    # glances/outputs/static/js/uiconfig.json file for the web interface
+    # This lidt can also be overwritten by the configuration file ([outputs] left_menu option)
     _left_sidebar = [
         'network',
         'ports',
@@ -122,8 +125,8 @@ class _GlancesCurses:
     _left_sidebar_min_width = 23
     _left_sidebar_max_width = 34
 
-    # Define right sidebar
-    _right_sidebar = ['containers', 'processcount', 'amps', 'processlist', 'alert']
+    # Define right sidebar in a method because it depends of self.args.programs
+    # See def _right_sidebar method
 
     def __init__(self, config=None, args=None):
         # Init
@@ -158,11 +161,11 @@ class _GlancesCurses:
         # Load configuration file
         self.load_config(config)
 
-        # Init cursor
-        self._init_cursor()
+        # Init Curses cursor
+        self._init_curses_cursor()
 
         # Init the colors
-        self._init_colors()
+        self.colors_list = GlancesColors(args).get()
 
         # Init main window
         self.term_window = self.screen.subwin(0, 0)
@@ -180,8 +183,10 @@ class _GlancesCurses:
         # Init the process min/max reset
         self.args.reset_minmax_tag = False
 
-        # Init cursor
+        # Init Glances cursor
         self.args.cursor_position = 0
+        # For the moment cursor only available in standalone mode
+        self.args.disable_cursor = not self.args.is_standalone
 
         # Catch key pressed with non blocking mode
         self.term_window.keypad(1)
@@ -195,17 +200,31 @@ class _GlancesCurses:
         """Load the outputs section of the configuration file."""
         if config is not None and config.has_section('outputs'):
             logger.debug('Read the outputs section in the configuration file')
-            # Separator ?
-            self.args.enable_separator = config.get_bool_value('outputs', 'separator', default=True)
+            # Separator
+            self.args.enable_separator = config.get_bool_value(
+                'outputs', 'separator', default=self.args.enable_separator
+            )
             # Set the left sidebar list
             self._left_sidebar = config.get_list_value('outputs', 'left_menu', default=self._left_sidebar)
+            # Background color
+            self.args.disable_bg = config.get_bool_value('outputs', 'disable_bg', default=self.args.disable_bg)
+
+    def _right_sidebar(self):
+        return [
+            'vms',
+            'containers',
+            'processcount',
+            'amps',
+            'programlist' if self.args.programs else 'processlist',
+            'alert',
+        ]
 
     def _init_history(self):
         """Init the history option."""
 
         self.reset_history_tag = False
 
-    def _init_cursor(self):
+    def _init_curses_cursor(self):
         """Init cursors."""
 
         if hasattr(curses, 'noecho'):
@@ -214,133 +233,6 @@ class _GlancesCurses:
             curses.cbreak()
         self.set_cursor(0)
 
-    def _init_colors(self):
-        """Init the Curses color layout."""
-
-        # Set curses options
-        try:
-            if hasattr(curses, 'start_color'):
-                curses.start_color()
-                logger.debug(f'Curses interface compatible with {curses.COLORS} colors')
-            if hasattr(curses, 'use_default_colors'):
-                curses.use_default_colors()
-        except Exception as e:
-            logger.warning(f'Error initializing terminal color ({e})')
-
-        # Init colors
-        if self.args.disable_bold:
-            A_BOLD = 0
-            self.args.disable_bg = True
-        else:
-            A_BOLD = curses.A_BOLD
-
-        self.title_color = A_BOLD
-        self.title_underline_color = A_BOLD | curses.A_UNDERLINE
-        self.help_color = A_BOLD
-
-        if curses.has_colors():
-            # The screen is compatible with a colored design
-            # ex: export TERM=xterm-256color
-            #     export TERM=xterm-color
-
-            curses.init_pair(1, -1, -1)
-            if self.args.disable_bg:
-                curses.init_pair(2, curses.COLOR_RED, -1)
-                curses.init_pair(3, curses.COLOR_GREEN, -1)
-                curses.init_pair(5, curses.COLOR_MAGENTA, -1)
-            else:
-                curses.init_pair(2, -1, curses.COLOR_RED)
-                curses.init_pair(3, -1, curses.COLOR_GREEN)
-                curses.init_pair(5, -1, curses.COLOR_MAGENTA)
-            curses.init_pair(4, curses.COLOR_BLUE, -1)
-            curses.init_pair(6, curses.COLOR_RED, -1)
-            curses.init_pair(7, curses.COLOR_GREEN, -1)
-            curses.init_pair(8, curses.COLOR_MAGENTA, -1)
-
-            # Colors text styles
-            self.no_color = curses.color_pair(1)
-            self.default_color = curses.color_pair(3) | A_BOLD
-            self.nice_color = curses.color_pair(8)
-            self.cpu_time_color = curses.color_pair(8)
-            self.ifCAREFUL_color = curses.color_pair(4) | A_BOLD
-            self.ifWARNING_color = curses.color_pair(5) | A_BOLD
-            self.ifCRITICAL_color = curses.color_pair(2) | A_BOLD
-            self.default_color2 = curses.color_pair(7)
-            self.ifCAREFUL_color2 = curses.color_pair(4)
-            self.ifWARNING_color2 = curses.color_pair(8) | A_BOLD
-            self.ifCRITICAL_color2 = curses.color_pair(6) | A_BOLD
-            self.ifINFO_color = curses.color_pair(4)
-            self.filter_color = A_BOLD
-            self.selected_color = A_BOLD
-            self.separator = curses.color_pair(1)
-
-            if curses.COLORS > 8:
-                # ex: export TERM=xterm-256color
-                colors_list = [curses.COLOR_CYAN, curses.COLOR_YELLOW]
-                for i in range(0, 3):
-                    try:
-                        curses.init_pair(i + 9, colors_list[i], -1)
-                    except Exception:
-                        curses.init_pair(i + 9, -1, -1)
-                self.filter_color = curses.color_pair(9) | A_BOLD
-                self.selected_color = curses.color_pair(10) | A_BOLD
-                # Define separator line style
-                try:
-                    curses.init_color(11, 500, 500, 500)
-                    curses.init_pair(11, curses.COLOR_BLACK, -1)
-                    self.separator = curses.color_pair(11)
-                except Exception:
-                    # Catch exception in TMUX
-                    pass
-        else:
-            # The screen is NOT compatible with a colored design
-            # switch to B&W text styles
-            # ex: export TERM=xterm-mono
-            self.no_color = -1
-            self.default_color = -1
-            self.nice_color = A_BOLD
-            self.cpu_time_color = A_BOLD
-            self.ifCAREFUL_color = A_BOLD
-            self.ifWARNING_color = curses.A_UNDERLINE
-            self.ifCRITICAL_color = curses.A_REVERSE
-            self.default_color2 = -1
-            self.ifCAREFUL_color2 = A_BOLD
-            self.ifWARNING_color2 = curses.A_UNDERLINE
-            self.ifCRITICAL_color2 = curses.A_REVERSE
-            self.ifINFO_color = A_BOLD
-            self.filter_color = A_BOLD
-            self.selected_color = A_BOLD
-            self.separator = -1
-
-        # Define the colors list (hash table) for stats
-        self.colors_list = {
-            'DEFAULT': self.no_color,
-            'UNDERLINE': curses.A_UNDERLINE,
-            'BOLD': A_BOLD,
-            'SORT': curses.A_UNDERLINE | A_BOLD,
-            'OK': self.default_color2,
-            'MAX': self.default_color2 | A_BOLD,
-            'FILTER': self.filter_color,
-            'TITLE': self.title_color,
-            'PROCESS': self.default_color2,
-            'PROCESS_SELECTED': self.default_color2 | curses.A_UNDERLINE,
-            'STATUS': self.default_color2,
-            'NICE': self.nice_color,
-            'CPU_TIME': self.cpu_time_color,
-            'CAREFUL': self.ifCAREFUL_color2,
-            'WARNING': self.ifWARNING_color2,
-            'CRITICAL': self.ifCRITICAL_color2,
-            'OK_LOG': self.default_color,
-            'CAREFUL_LOG': self.ifCAREFUL_color,
-            'WARNING_LOG': self.ifWARNING_color,
-            'CRITICAL_LOG': self.ifCRITICAL_color,
-            'PASSWORD': curses.A_PROTECT,
-            'SELECTED': self.selected_color,
-            'INFO': self.ifINFO_color,
-            'ERROR': self.selected_color,
-            'SEPARATOR': self.separator,
-        }
-
     def set_cursor(self, value):
         """Configure the curse cursor appearance.
 
@@ -358,40 +250,39 @@ class _GlancesCurses:
         # TODO: Check issue #163
         return window.getch()
 
+    def catch_actions_from_hotkey(self, hotkey):
+        if self.pressedkey == ord(hotkey) and 'switch' in self._hotkeys[hotkey]:
+            self._handle_switch(hotkey)
+        elif self.pressedkey == ord(hotkey) and 'sort_key' in self._hotkeys[hotkey]:
+            self._handle_sort_key(hotkey)
+        if self.pressedkey == ord(hotkey) and 'handler' in self._hotkeys[hotkey]:
+            action = getattr(self, self._hotkeys[hotkey]['handler'])
+            action()
+
+    def catch_other_actions_maybe_return_to_browser(self, return_to_browser):
+        {
+            self.pressedkey in {ord('e')} and not self.args.programs: self._handle_process_extended,
+            self.pressedkey in {ord('k')} and not self.args.disable_cursor: self._handle_kill_process,
+            self.pressedkey in {curses.KEY_LEFT}: self._handle_sort_left,
+            self.pressedkey in {curses.KEY_RIGHT}: self._handle_sort_right,
+            self.pressedkey in {curses.KEY_UP, 65} and not self.args.disable_cursor: self._handle_cursor_up,
+            self.pressedkey in {curses.KEY_DOWN, 66} and not self.args.disable_cursor: self._handle_cursor_down,
+            self.pressedkey in {curses.KEY_F5, 18}: self._handle_refresh,
+            self.pressedkey in {ord('\x1b'), ord('q')}: functools.partial(self._handle_quit, return_to_browser),
+        }.get(True, lambda: None)()
+
     def __catch_key(self, return_to_browser=False):
         # Catch the pressed key
         self.pressedkey = self.get_key(self.term_window)
         if self.pressedkey == -1:
-            return -1
+            return self.pressedkey
 
         # Actions (available in the global hotkey dict)...
         logger.debug(f"Keypressed (code: {self.pressedkey})")
-        for hotkey in self._hotkeys:
-            if self.pressedkey == ord(hotkey) and 'switch' in self._hotkeys[hotkey]:
-                self._handle_switch(hotkey)
-            elif self.pressedkey == ord(hotkey) and 'sort_key' in self._hotkeys[hotkey]:
-                self._handle_sort_key(hotkey)
-            if self.pressedkey == ord(hotkey) and 'handler' in self._hotkeys[hotkey]:
-                action = getattr(self, self._hotkeys[hotkey]['handler'])
-                action()
+        [self.catch_actions_from_hotkey(hotkey) for hotkey in self._hotkeys]
 
         # Other actions with key > 255 (ord will not work) and/or additional test...
-        if self.pressedkey == ord('e') and not self.args.programs:
-            self._handle_process_extended()
-        elif self.pressedkey == ord('k') and not self.args.disable_cursor:
-            self._handle_kill_process()
-        elif self.pressedkey == curses.KEY_LEFT:
-            self._handle_sort_left()
-        elif self.pressedkey == curses.KEY_RIGHT:
-            self._handle_sort_right()
-        elif self.pressedkey == curses.KEY_UP or self.pressedkey == 65 and not self.args.disable_cursor:
-            self._handle_cursor_up()
-        elif self.pressedkey == curses.KEY_DOWN or self.pressedkey == 66 and not self.args.disable_cursor:
-            self._handle_cursor_down()
-        elif self.pressedkey == ord('\x1b') or self.pressedkey == ord('q'):
-            self._handle_quit(return_to_browser)
-        elif self.pressedkey == curses.KEY_F5 or self.pressedkey == 18:
-            self._handle_refresh()
+        self.catch_other_actions_maybe_return_to_browser(return_to_browser)
 
         # Return the key code
         return self.pressedkey
@@ -495,7 +386,7 @@ class _GlancesCurses:
             logger.info(f"Stop Glances (keypressed: {self.pressedkey})")
 
     def _handle_refresh(self):
-        pass
+        glances_processes.reset_internal_cache()
 
     def loop_position(self):
         """Return the current sort in the loop"""
@@ -570,13 +461,16 @@ class _GlancesCurses:
         self.new_line()
         self.line -= 1
         line_width = self.term_window.getmaxyx()[1] - self.column
-        self.term_window.addnstr(
-            self.line,
-            self.column,
-            unicode_message('MEDIUM_LINE', self.args) * line_width,
-            line_width,
-            self.colors_list[color],
-        )
+        if self.line >= 0 and self.line < self.term_window.getmaxyx()[0]:
+            position = [self.line, self.column]
+            line_color = self.colors_list[color]
+            line_type = curses.ACS_HLINE if not self.args.disable_unicode else unicode_message('MEDIUM_LINE', self.args)
+            self.term_window.hline(
+                *position,
+                line_type,
+                line_width,
+                line_color,
+            )
 
     def __get_stat_display(self, stats, layer):
         """Return a dict of dict with all the stats display.
@@ -775,11 +669,10 @@ class _GlancesCurses:
             self.new_column()
             self.display_plugin(stat_display["ip"], display_optional=(self.term_window.getmaxyx()[1] >= 100))
         self.new_column()
-        self.display_plugin(
-            stat_display["uptime"], add_space=-(self.get_stats_display_width(stat_display["cloud"]) != 0)
-        )
+        cloud_width = self.get_stats_display_width(stat_display.get("cloud", 0))
+        self.display_plugin(stat_display["uptime"], add_space=-(cloud_width != 0))
         self.init_column()
-        if self.get_stats_display_width(stat_display["cloud"]) != 0:
+        if cloud_width != 0:
             # Second line (optional)
             self.new_line()
             self.display_plugin(stat_display["cloud"])
@@ -803,7 +696,7 @@ class _GlancesCurses:
             )
 
         # Width of all plugins
-        stats_width = sum(itervalues(plugin_widths))
+        stats_width = sum(plugin_widths.values())
 
         # Number of plugin but quicklook
         stats_number = sum(
@@ -829,7 +722,7 @@ class _GlancesCurses:
                 logger.debug(f"Quicklook plugin not available ({e})")
             else:
                 plugin_widths['quicklook'] = self.get_stats_display_width(stat_display["quicklook"])
-                stats_width = sum(itervalues(plugin_widths)) + 1
+                stats_width = sum(plugin_widths.values()) + 1
             self.space_between_column = 1
             self.display_plugin(stat_display["quicklook"])
             self.new_column()
@@ -850,7 +743,7 @@ class _GlancesCurses:
                         if hasattr(self.args, 'disable_' + p)
                         else 0
                     )
-                    stats_width = sum(itervalues(plugin_widths)) + 1
+                    stats_width = sum(plugin_widths.values()) + 1
                     self.space_between_column = max(
                         1, int((self.term_window.getmaxyx()[1] - stats_width) / (stats_number - 1))
                     )
@@ -905,16 +798,24 @@ class _GlancesCurses:
 
         # Display right sidebar
         self.new_column()
-        for p in self._right_sidebar:
+        for p in self._right_sidebar():
             if (hasattr(self.args, 'enable_' + p) or hasattr(self.args, 'disable_' + p)) and p in stat_display:
                 self.new_line()
-                if p == 'processlist':
+                if p in ['processlist', 'programlist']:
+                    p_index = self._right_sidebar().index(p) + 1
                     self.display_plugin(
-                        stat_display['processlist'],
+                        stat_display[p],
                         display_optional=(self.term_window.getmaxyx()[1] > 102),
                         display_additional=(not MACOS),
                         max_y=(
-                            self.term_window.getmaxyx()[0] - self.get_stats_display_height(stat_display['alert']) - 2
+                            self.term_window.getmaxyx()[0]
+                            - sum(
+                                [
+                                    self.get_stats_display_height(stat_display[i])
+                                    for i in self._right_sidebar()[p_index:]
+                                ]
+                            )
+                            - 2
                         ),
                     )
                 else:
@@ -978,7 +879,7 @@ class _GlancesCurses:
 
         # Add the message
         for y, m in enumerate(sentence_list):
-            if len(m) > 0:
+            if m:
                 popup.addnstr(2 + y, 2, m, len(m))
 
         if popup_type == 'info':
@@ -1018,11 +919,14 @@ class _GlancesCurses:
             return None
 
         if popup_type == 'yesno':
-            # # Create a sub-window for the text field
+            # Create a sub-window for the text field
             sub_pop = popup.derwin(1, 2, len(sentence_list) + 1, len(m) + 2)
             sub_pop.attron(self.colors_list['FILTER'])
             # Init the field with the current value
-            sub_pop.addnstr(0, 0, '', 0)
+            try:
+                sub_pop.addnstr(0, 0, '', 0)
+            except curses.error:
+                pass
             # Display the popup
             popup.refresh()
             sub_pop.refresh()
@@ -1037,6 +941,85 @@ class _GlancesCurses:
 
         return None
 
+    def setup_upper_left_pos(self, plugin_stats):
+        screen_y, screen_x = self.term_window.getmaxyx()
+
+        if plugin_stats['align'] == 'right':
+            # Right align (last column)
+            display_x = screen_x - self.get_stats_display_width(plugin_stats)
+        else:
+            display_x = self.column
+
+        if plugin_stats['align'] == 'bottom':
+            # Bottom (last line)
+            display_y = screen_y - self.get_stats_display_height(plugin_stats)
+        else:
+            display_y = self.line
+
+        return display_y, display_x
+
+    def get_next_x_and_x_max(self, m, x, x_max):
+        # New column
+        # Python 2: we need to decode to get real screen size because
+        # UTF-8 special tree chars occupy several bytes.
+        # Python 3: strings are strings and bytes are bytes, all is
+        # good.
+        try:
+            x += len(u(m['msg']))
+        except UnicodeDecodeError:
+            # Quick and dirty hack for issue #745
+            pass
+        if x > x_max:
+            x_max = x
+
+        return x, x_max
+
+    def display_stats_with_current_size(self, m, y, x):
+        screen_x = self.term_window.getmaxyx()[1]
+        self.term_window.addnstr(
+            y,
+            x,
+            m['msg'],
+            # Do not display outside the screen
+            screen_x - x,
+            self.colors_list[m['decoration']],
+        )
+
+    def display_stats(self, plugin_stats, init, helper):
+        y, x, x_max = init
+        for m in plugin_stats['msgdict']:
+            # New line
+            try:
+                if m['msg'].startswith('\n'):
+                    y, x = helper['goto next, add first col'](y, x)
+                    continue
+            except Exception:
+                # Avoid exception (see issue #1692)
+                pass
+            # Do not display outside the screen
+            if x < 0:
+                continue
+            if helper['x overbound?'](m, x):
+                continue
+            if helper['y overbound?'](y):
+                break
+            # If display_optional = False do not display optional stats
+            if helper['display optional?'](m):
+                continue
+            # If display_additional = False do not display additional stats
+            if helper['display additional?'](m):
+                continue
+            # Is it possible to display the stat with the current screen size
+            # !!! Crash if not try/except... Why ???
+            try:
+                self.display_stats_with_current_size(m, y, x)
+            except Exception:
+                pass
+            else:
+                x, x_max = self.get_next_x_and_x_max(m, x, x_max)
+
+        return y, x, x_max
+
     def display_plugin(self, plugin_stats, display_optional=True, display_additional=True, max_y=65535, add_space=0):
         """Display the plugin_stats on the screen.
 
@@ -1054,76 +1037,22 @@ class _GlancesCurses:
             return 0
 
         # Get the screen size
-        screen_x = self.term_window.getmaxyx()[1]
-        screen_y = self.term_window.getmaxyx()[0]
+        screen_y, screen_x = self.term_window.getmaxyx()
 
         # Set the upper/left position of the message
-        if plugin_stats['align'] == 'right':
-            # Right align (last column)
-            display_x = screen_x - self.get_stats_display_width(plugin_stats)
-        else:
-            display_x = self.column
-        if plugin_stats['align'] == 'bottom':
-            # Bottom (last line)
-            display_y = screen_y - self.get_stats_display_height(plugin_stats)
-        else:
-            display_y = self.line
+        display_y, display_x = self.setup_upper_left_pos(plugin_stats)
+
+        helper = {
+            'goto next, add first col': lambda y, x: (y + 1, display_x),
+            'x overbound?': lambda m, x: not m['splittable'] and (x + len(m['msg']) > screen_x),
+            'y overbound?': lambda y: y < 0 or (y + 1 > screen_y) or (y > max_y),
+            'display optional?': lambda m: not display_optional and m['optional'],
+            'display additional?': lambda m: not display_additional and m['additional'],
+        }
 
         # Display
-        x = display_x
-        x_max = x
-        y = display_y
-        for m in plugin_stats['msgdict']:
-            # New line
-            try:
-                if m['msg'].startswith('\n'):
-                    # Go to the next line
-                    y += 1
-                    # Return to the first column
-                    x = display_x
-                    continue
-            except Exception:
-                # Avoid exception (see issue #1692)
-                pass
-            # Do not display outside the screen
-            if x < 0:
-                continue
-            if not m['splittable'] and (x + len(m['msg']) > screen_x):
-                continue
-            if y < 0 or (y + 1 > screen_y) or (y > max_y):
-                break
-            # If display_optional = False do not display optional stats
-            if not display_optional and m['optional']:
-                continue
-            # If display_additional = False do not display additional stats
-            if not display_additional and m['additional']:
-                continue
-            # Is it possible to display the stat with the current screen size
-            # !!! Crash if not try/except... Why ???
-            try:
-                self.term_window.addnstr(
-                    y,
-                    x,
-                    m['msg'],
-                    # Do not display outside the screen
-                    screen_x - x,
-                    self.colors_list[m['decoration']],
-                )
-            except Exception:
-                pass
-            else:
-                # New column
-                # Python 2: we need to decode to get real screen size because
-                # UTF-8 special tree chars occupy several bytes.
-                # Python 3: strings are strings and bytes are bytes, all is
-                # good.
-                try:
-                    x += len(u(m['msg']))
-                except UnicodeDecodeError:
-                    # Quick and dirty hack for issue #745
-                    pass
-                if x > x_max:
-                    x_max = x
+        init = display_y, display_x, display_x
+        y, x, x_max = self.display_stats(plugin_stats, init, helper)
 
         # Compute the next Glances column/line position
         self.next_column = max(self.next_column, x_max + self.space_between_column)
@@ -1277,10 +1206,17 @@ class _GlancesCurses:
 class GlancesCursesStandalone(_GlancesCurses):
     """Class for the Glances curse standalone."""
 
+    # Default number of processes to displayed is set to 50
+    glances_processes.max_processes = 50
+
 
 class GlancesCursesClient(_GlancesCurses):
     """Class for the Glances curse client."""
 
+    # Default number of processes to displayed is set to 50
+    # For the moment, cursor in client/server mode is not supported see #3221
+    glances_processes.max_processes = 50
+
 
 class GlancesTextbox(Textbox):
     def __init__(self, *args, **kwargs):
