@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # This file is part of Glances.
 #
@@ -14,7 +13,7 @@ import time
 from glances.globals import printandflush
 
 
-class GlancesStdoutJson(object):
+class GlancesStdoutJson:
     """This class manages the Stdout JSON display."""
 
     def __init__(self, config=None, args=None):
@@ -35,19 +34,25 @@ class GlancesStdoutJson(object):
     def end(self):
         pass
 
-    def update(self, stats, duration=3):
+    def update(self, stats, duration=3, cs_status=None, return_to_browser=False):
         """Display stats in JSON format to stdout.
 
         Refresh every duration second.
         """
+        all_in_json = '{'
+        plugins_list_json = []
         for plugin in self.plugins_list:
             # Check if the plugin exist and is enable
             if plugin in stats.getPluginsList() and stats.get_plugin(plugin).is_enabled():
-                stat = stats.get_plugin(plugin).get_json()
+                plugins_list_json.append(f'"{plugin}": {stats.get_plugin(plugin).get_json().decode("utf-8")}')
             else:
                 continue
-            # Display stats
-            printandflush('{}: {}'.format(plugin, stat))
+        # Join all plugins in a single JSON object
+        all_in_json += ', '.join(plugins_list_json)
+        all_in_json += '}'
+
+        # Display stats
+        printandflush(all_in_json)
 
         # Wait until next refresh
         if duration > 0:
