@@ -11,15 +11,26 @@ import os
 import psutil
 
 from glances.filter import GlancesFilter, GlancesFilterList
-from glances.globals import BSD, LINUX, MACOS, WINDOWS, iterkeys, list_of_namedtuple_to_list_of_dict, namedtuple_to_dict
+from glances.globals import (
+    BSD,
+    LINUX,
+    MACOS,
+    WINDOWS,
+    dictlist_first_key_value,
+    list_of_namedtuple_to_list_of_dict,
+    namedtuple_to_dict,
+)
 from glances.logger import logger
 from glances.programs import processes_to_programs
 from glances.timer import Timer, getTimeSinceLastUpdate
 
 psutil_version_info = tuple([int(num) for num in psutil.__version__.split('.')])
 
+# This constant defines the list of mandatory processes stats. Thoses stats can not be disabled by the user
+mandatory_processes_stats_list = ['pid', 'name']
+
 # This constant defines the list of available processes sort key
-sort_processes_key_list = ['cpu_percent', 'memory_percent', 'username', 'cpu_times', 'io_counters', 'name']
+sort_processes_stats_list = ['cpu_percent', 'memory_percent', 'username', 'cpu_times', 'io_counters', 'name']
 
 # Sort dictionary for human
 sort_for_human = {
@@ -38,7 +49,7 @@ class GlancesProcesses:
 
     def __init__(self, cache_timeout=60):
         """Init the class to collect stats about processes."""
-        # Init the args, coming from the GlancesStandalone class
+        # Init the args, coming from the classes derived from GlancesMode
         # Should be set by the set_args method
         self.args = None
 
@@ -77,6 +88,30 @@ class GlancesProcesses:
         self.disable_extended_tag = False
         self.extended_process = None
 
+        # Tests (and disable if not available) optionals features
+        self._test_grab()
+
+        # Maximum number of processes showed in the UI (None if no limit)
+        self._max_processes = None
+
+        # Process filter
+        self._filter = GlancesFilter()
+
+        # Whether or not to hide kernel threads
+        self.no_kernel_threads = False
+
+        # Store maximums values in a dict
+        # Used in the UI to highlight the maximum value
+        self._max_values_list = ('cpu_percent', 'memory_percent')
+        # { 'cpu_percent': 0.0, 'memory_percent': 0.0 }
+        self._max_values = {}
+        self.reset_max_values()
+
+        # Set the key's list be disabled in order to only display specific attribute in the process list
+        self.disable_stats = []
+
+    def _test_grab(self):
+        """Test somes optionals features"""
         # Test if the system can grab io_counters
         try:
             p = psutil.Process()
@@ -99,26 +134,18 @@ class GlancesProcesses:
             logger.debug('PsUtil can grab processes gids')
             self.disable_gids = False
 
-        # Maximum number of processes showed in the UI (None if no limit)
-        self._max_processes = None
-
-        # Process filter
-        self._filter = GlancesFilter()
-
-        # Whether or not to hide kernel threads
-        self.no_kernel_threads = False
-
-        # Store maximums values in a dict
-        # Used in the UI to highlight the maximum value
-        self._max_values_list = ('cpu_percent', 'memory_percent')
-        # { 'cpu_percent': 0.0, 'memory_percent': 0.0 }
-        self._max_values = {}
-        self.reset_max_values()
-
     def set_args(self, args):
         """Set args."""
         self.args = args
 
+    def reset_internal_cache(self):
+        """Reset the internal cache."""
+        self.cache_timer = Timer(0)
+        self.processlist_cache = {}
+        if hasattr(psutil.process_iter, 'cache_clear'):
+            # Cache clear only available in PsUtil 6 or higher
+            psutil.process_iter.cache_clear()
+
     def reset_processcount(self):
         """Reset the global process count"""
         self.processcount = {'total': 0, 'running': 0, 'sleeping': 0, 'thread': 0, 'pid_max': None}
@@ -129,10 +156,13 @@ class GlancesProcesses:
         self.processcount['pid_max'] = self.pid_max
         # For each key in the processcount dict
         # count the number of processes with the same status
-        for k in iterkeys(self.processcount):
-            self.processcount[k] = len(list(filter(lambda v: v['status'] is k, plist)))
+        for k in list(self.processcount.keys()):
+            self.processcount[k] = len(list(filter(lambda v: v.get('status', '?') is k, plist)))
         # Compute thread
-        self.processcount['thread'] = sum(i['num_threads'] for i in plist if i['num_threads'] is not None)
+        try:
+            self.processcount['thread'] = sum(i['num_threads'] for i in plist if i['num_threads'] is not None)
+        except KeyError:
+            self.processcount['thread'] = None
         # Compute total
         self.processcount['total'] = len(plist)
 
@@ -202,7 +232,15 @@ class GlancesProcesses:
         """Set the maximum number of processes showed in the UI."""
         self._max_processes = value
 
-    # Process filter
+    @property
+    def disable_stats(self):
+        """Set disable_stats list"""
+        return self._disable_stats
+
+    @disable_stats.setter
+    def disable_stats(self, stats_list):
+        """Set disable_stats list"""
+        self._disable_stats = [i for i in stats_list if i not in mandatory_processes_stats_list]
 
     @property
     def process_filter_input(self):
@@ -273,8 +311,8 @@ class GlancesProcesses:
         for k in self._max_values_list:
             self._max_values[k] = 0.0
 
-    def get_extended_stats(self, proc):
-        """Get the extended stats for the given PID."""
+    def set_extended_stats(self, proc):
+        """Set the extended stats for the given PID."""
         # - cpu_affinity (Linux, Windows, FreeBSD)
         # - ionice (Linux and Windows > Vista)
         # - num_ctx_switches (not available on Illumos/Solaris)
@@ -285,7 +323,7 @@ class GlancesProcesses:
         # - connections (TCP and UDP)
         # - CPU min/max/mean
 
-        # Set the extended stats list (OS dependant)
+        # Set the extended stats list (OS dependent)
         extended_stats = ['cpu_affinity', 'ionice', 'num_ctx_switches']
         if LINUX:
             # num_fds only available on Unix system (see issue #1351)
@@ -320,6 +358,16 @@ class GlancesProcesses:
             ret['extended_stats'] = True
         return namedtuple_to_dict(ret)
 
+    def get_extended_stats(self):
+        """Return the extended stats.
+
+        Return the process stat when extended_stats = True
+        """
+        for p in self.processlist:
+            if p.get('extended_stats'):
+                return p
+        return None
+
     def __get_min_max_mean(self, proc, prefix=['cpu', 'memory']):
         """Return the min/max/mean for the given process"""
         ret = {}
@@ -394,6 +442,118 @@ class GlancesProcesses:
             and not self.args.disable_cursor
         )
 
+    def build_process_list(self, sorted_attrs):
+        # Build the processes stats list (it is why we need psutil>=5.3.0) (see issue #2755)
+        processlist = list(
+            filter(
+                lambda p: not (BSD and p.info['name'] == 'idle')
+                and not (WINDOWS and p.info['name'] == 'System Idle Process')
+                and not (MACOS and p.info['name'] == 'kernel_task')
+                and not (self.no_kernel_threads and LINUX and p.info['gids'].real == 0),
+                psutil.process_iter(attrs=sorted_attrs, ad_value=None),
+            )
+        )
+
+        # Only get the info key
+        # PsUtil 6+ no longer check PID reused #2755 so use is_running in the loop
+        # Note: not sure it is realy needed but CPU consumption look the same with or without it
+        processlist = [p.info for p in processlist if p.is_running()]
+
+        # Sort the processes list by the current sort_key
+        return sort_stats(processlist, sorted_by=self.sort_key, reverse=True)
+
+    def get_sorted_attrs(self):
+        defaults = ['cpu_percent', 'cpu_times', 'memory_percent', 'name', 'status', 'num_threads']
+        optional = ['io_counters'] if not self.disable_io_counters else []
+
+        return defaults + optional
+
+    def get_displayed_attr(self):
+        defaults = ['memory_info', 'nice', 'pid']
+        optional = ['gids'] if not self.disable_gids else []
+
+        return defaults + optional
+
+    def get_cached_attrs(self):
+        return ['cmdline', 'username']
+
+    def maybe_add_cached_attrs(self, sorted_attrs, cached_attrs):
+        # Some stats are not sort key
+        # An optimisation can be done be only grabbed displayed_attr
+        # for displayed processes (but only in standalone mode...)
+        sorted_attrs.extend(self.get_displayed_attr())
+        # Some stats are cached (not necessary to be refreshed every time)
+        if self.cache_timer.finished():
+            sorted_attrs += cached_attrs
+            self.cache_timer.set(self.cache_timeout)
+            self.cache_timer.reset()
+            is_cached = False
+        else:
+            is_cached = True
+
+        return is_cached, sorted_attrs
+
+    def get_pid_time_and_status(self, time_since_update, proc):
+        # PID is the key
+        proc['key'] = 'pid'
+
+        # Time since last update (for disk_io rate computation)
+        proc['time_since_update'] = time_since_update
+
+        # Process status (only keep the first char)
+        proc['status'] = str(proc.get('status', '?'))[:1].upper()
+
+        return proc
+
+    def get_io_counters(self, proc):
+        # procstat['io_counters'] is a list:
+        # [read_bytes, write_bytes, read_bytes_old, write_bytes_old, io_tag]
+        # If io_tag = 0 > Access denied or first time (display "?")
+        # If io_tag = 1 > No access denied (display the IO rate)
+        if 'io_counters' in proc and proc['io_counters'] is not None:
+            io_new = [proc['io_counters'][2], proc['io_counters'][3]]
+            # For IO rate computation
+            # Append saved IO r/w bytes
+            try:
+                proc['io_counters'] = io_new + self.io_old[proc['pid']]
+                io_tag = 1
+            except KeyError:
+                proc['io_counters'] = io_new + [0, 0]
+                io_tag = 0
+            # then save the IO r/w bytes
+            self.io_old[proc['pid']] = io_new
+        else:
+            proc['io_counters'] = [0, 0] + [0, 0]
+            io_tag = 0
+        # Append the IO tag (for display)
+        proc['io_counters'] += [io_tag]
+
+        return proc
+
+    def maybe_add_cached_stats(self, is_cached, cached_attrs, proc):
+        if is_cached:
+            # Grab cached values (in case of a new incoming process)
+            if proc['pid'] not in self.processlist_cache:
+                try:
+                    self.processlist_cache[proc['pid']] = psutil.Process(pid=proc['pid']).as_dict(
+                        attrs=cached_attrs, ad_value=None
+                    )
+                except psutil.NoSuchProcess:
+                    pass
+            # Add cached value to current stat
+            try:
+                proc.update(self.processlist_cache[proc['pid']])
+            except KeyError:
+                pass
+        else:
+            # Save values to cache
+            try:
+                self.processlist_cache[proc['pid']] = {cached: proc[cached] for cached in cached_attrs}
+            except KeyError:
+                pass
+
+        return proc
+
     def update(self):
         """Update the processes stats."""
         # Init new processes stats
@@ -408,46 +568,17 @@ class GlancesProcesses:
 
         # Grab standard stats
         #####################
-        sorted_attrs = ['cpu_percent', 'cpu_times', 'memory_percent', 'name', 'status', 'num_threads']
-        displayed_attr = ['memory_info', 'nice', 'pid']
-        # The following attributes are cached and only retreive every self.cache_timeout seconds
+        sorted_attrs = self.get_sorted_attrs()
+
+        # The following attributes are cached and only retrieve every self.cache_timeout seconds
         # Warning: 'name' can not be cached because it is used for filtering
-        cached_attrs = ['cmdline', 'username']
+        cached_attrs = self.get_cached_attrs()
 
-        # Some stats are optional
-        if not self.disable_io_counters:
-            sorted_attrs.append('io_counters')
-        if not self.disable_gids:
-            displayed_attr.append('gids')
-        # Some stats are not sort key
-        # An optimisation can be done be only grabbed displayed_attr
-        # for displayed processes (but only in standalone mode...)
-        sorted_attrs.extend(displayed_attr)
-        # Some stats are cached (not necessary to be refreshed every time)
-        if self.cache_timer.finished():
-            sorted_attrs += cached_attrs
-            self.cache_timer.set(self.cache_timeout)
-            self.cache_timer.reset()
-            is_cached = False
-        else:
-            is_cached = True
+        is_cached, sorted_attrs = self.maybe_add_cached_attrs(sorted_attrs, cached_attrs)
 
-        # Build the processes stats list (it is why we need psutil>=5.3.0)
-        # This is one of the main bottleneck of Glances (see flame graph)
-        # It may be optimized with PsUtil 6+ (see issue #2755)
-        processlist = list(
-            filter(
-                lambda p: not (BSD and p.info['name'] == 'idle')
-                and not (WINDOWS and p.info['name'] == 'System Idle Process')
-                and not (MACOS and p.info['name'] == 'kernel_task')
-                and not (self.no_kernel_threads and LINUX and p.info['gids'].real == 0),
-                psutil.process_iter(attrs=sorted_attrs, ad_value=None),
-            )
-        )
-        # Only get the info key
-        processlist = [p.info for p in processlist]
-        # Sort the processes list by the current sort_key
-        processlist = sort_stats(processlist, sorted_by=self.sort_key, reverse=True)
+        # Remove attributes set by the user in the config file (see #1524)
+        sorted_attrs = [i for i in sorted_attrs if i not in self.disable_stats]
+        processlist = self.build_process_list(sorted_attrs)
 
         # Update the processcount
         self.update_processcount(processlist)
@@ -465,65 +596,21 @@ class GlancesProcesses:
 
             # Grab extended stats only for the selected process (see issue #2225)
             if self.extended_process is not None and proc['pid'] == self.extended_process['pid']:
-                proc.update(self.get_extended_stats(self.extended_process))
+                proc.update(self.set_extended_stats(self.extended_process))
                 self.extended_process = namedtuple_to_dict(proc)
 
             # Meta data
             ###########
-
-            # PID is the key
-            proc['key'] = 'pid'
-
-            # Time since last update (for disk_io rate computation)
-            proc['time_since_update'] = time_since_update
-
-            # Process status (only keep the first char)
-            proc['status'] = str(proc['status'])[:1].upper()
+            proc = self.get_pid_time_and_status(time_since_update, proc)
 
             # Process IO
-            # procstat['io_counters'] is a list:
-            # [read_bytes, write_bytes, read_bytes_old, write_bytes_old, io_tag]
-            # If io_tag = 0 > Access denied or first time (display "?")
-            # If io_tag = 1 > No access denied (display the IO rate)
-            if 'io_counters' in proc and proc['io_counters'] is not None:
-                io_new = [proc['io_counters'][2], proc['io_counters'][3]]
-                # For IO rate computation
-                # Append saved IO r/w bytes
-                try:
-                    proc['io_counters'] = io_new + self.io_old[proc['pid']]
-                    io_tag = 1
-                except KeyError:
-                    proc['io_counters'] = io_new + [0, 0]
-                    io_tag = 0
-                # then save the IO r/w bytes
-                self.io_old[proc['pid']] = io_new
-            else:
-                proc['io_counters'] = [0, 0] + [0, 0]
-                io_tag = 0
-            # Append the IO tag (for display)
-            proc['io_counters'] += [io_tag]
+            proc = self.get_io_counters(proc)
 
             # Manage cached information
-            if is_cached:
-                # Grab cached values (in case of a new incoming process)
-                if proc['pid'] not in self.processlist_cache:
-                    try:
-                        self.processlist_cache[proc['pid']] = psutil.Process(pid=proc['pid']).as_dict(
-                            attrs=cached_attrs, ad_value=None
-                        )
-                    except psutil.NoSuchProcess:
-                        pass
-                # Add cached value to current stat
-                try:
-                    proc.update(self.processlist_cache[proc['pid']])
-                except KeyError:
-                    pass
-            else:
-                # Save values to cache
-                try:
-                    self.processlist_cache[proc['pid']] = {cached: proc[cached] for cached in cached_attrs}
-                except KeyError:
-                    pass
+            proc = self.maybe_add_cached_stats(is_cached, cached_attrs, proc)
+
+        # Remove non running process from the cache (avoid issue #2976)
+        self.remove_non_running_procs(processlist)
 
         # Filter and transform process export list
         self.processlist_export = self.update_export_list(processlist)
@@ -533,16 +620,26 @@ class GlancesProcesses:
 
         # Compute the maximum value for keys in self._max_values_list: CPU, MEM
         # Useful to highlight the processes with maximum values
-        for k in self._max_values_list:
-            values_list = [i[k] for i in processlist if i[k] is not None]
-            if values_list:
-                self.set_max_values(k, max(values_list))
+        self.compute_max_value(processlist)
 
         # Update the stats
         self.processlist = processlist
 
         return self.processlist
 
+    def compute_max_value(self, processlist):
+        for k in [i for i in self._max_values_list if i not in self.disable_stats]:
+            values_list = [i[k] for i in processlist if i[k] is not None]
+            if values_list:
+                self.set_max_values(k, max(values_list))
+
+    def remove_non_running_procs(self, processlist):
+        pids_running = [p['pid'] for p in processlist]
+        pids_cached = list(self.processlist_cache.keys()).copy()
+        for pid in pids_cached:
+            if pid not in pids_running:
+                self.processlist_cache.pop(pid, None)
+
     def update_list(self, processlist):
         """Return the process list after filtering and transformation (namedtuple to dict)."""
         if self._filter.filter is None:
@@ -573,6 +670,10 @@ class GlancesProcesses:
         """Return the processlist for export."""
         return self.processlist_export
 
+    def get_stats(self, pid):
+        """Get stats for the given pid."""
+        return dictlist_first_key_value(self.processlist, 'pid', pid)
+
     @property
     def sort_key(self):
         """Get the current sort key."""
@@ -623,6 +724,10 @@ def weighted(value):
     return -float('inf') if value is None else value
 
 
+def sort_by_these_keys(first, second):
+    return lambda process: (weighted(process.get(first)), weighted(process.get(second)))
+
+
 def _sort_io_counters(process, sorted_by='io_counters', sorted_by_secondary='memory_percent'):
     """Specific case for io_counters
 
@@ -644,12 +749,7 @@ def _sort_cpu_times(process, sorted_by='cpu_times', sorted_by_secondary='memory_
 
 def _sort_lambda(sorted_by='cpu_percent', sorted_by_secondary='memory_percent'):
     """Return a sort lambda function for the sorted_by key"""
-    ret = None
-    if sorted_by == 'io_counters':
-        ret = _sort_io_counters
-    elif sorted_by == 'cpu_times':
-        ret = _sort_cpu_times
-    return ret
+    return {'io_counters': _sort_io_counters, 'cpu_times': _sort_cpu_times}.get(sorted_by, None)
 
 
 def sort_stats(stats, sorted_by='cpu_percent', sorted_by_secondary='memory_percent', reverse=True):
@@ -668,20 +768,14 @@ def sort_stats(stats, sorted_by='cpu_percent', sorted_by_secondary='memory_perce
     if sort_lambda is not None:
         # Specific sort
         try:
-            stats.sort(key=sort_lambda, reverse=reverse)
+            stats = sorted(stats, key=sort_lambda, reverse=reverse)
         except Exception:
             # If an error is detected, fallback to cpu_percent
-            stats.sort(
-                key=lambda process: (weighted(process['cpu_percent']), weighted(process[sorted_by_secondary])),
-                reverse=reverse,
-            )
+            stats = sorted(stats, key=sort_by_these_keys('cpu_percent', sorted_by_secondary), reverse=reverse)
     else:
         # Standard sort
         try:
-            stats.sort(
-                key=lambda process: (weighted(process[sorted_by]), weighted(process[sorted_by_secondary])),
-                reverse=reverse,
-            )
+            stats = sorted(stats, key=sort_by_these_keys(sorted_by, sorted_by_secondary), reverse=reverse)
         except (KeyError, TypeError):
             # Fallback to name
             stats.sort(key=lambda process: process['name'] if process['name'] is not None else '~', reverse=False)
