@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # This file is part of Glances.
 #
@@ -13,7 +12,7 @@ from glances.globals import urlparse
 from glances.logger import logger
 
 
-class GlancesWebList(object):
+class GlancesWebList:
     """Manage the Web/Url list for the ports plugin."""
 
     _section = "ports"
@@ -36,9 +35,9 @@ class GlancesWebList(object):
         if config is None:
             logger.debug("No configuration file available. Cannot load ports list.")
         elif not config.has_section(self._section):
-            logger.debug("No [%s] section in the configuration file. Cannot load ports list." % self._section)
+            logger.debug(f"No [{self._section}] section in the configuration file. Cannot load ports list.")
         else:
-            logger.debug("Start reading the [%s] section in the configuration file" % self._section)
+            logger.debug(f"Start reading the [{self._section}] section in the configuration file")
 
             refresh = int(config.get_value(self._section, 'refresh', default=self._default_refresh))
             timeout = int(config.get_value(self._section, 'timeout', default=self._default_timeout))
@@ -46,23 +45,23 @@ class GlancesWebList(object):
             # Read the web/url list
             for i in range(1, 256):
                 new_web = {}
-                postfix = 'web_%s_' % str(i)
+                postfix = f'web_{str(i)}_'
 
                 # Read mandatory configuration key: host
-                new_web['url'] = config.get_value(self._section, '%s%s' % (postfix, 'url'))
+                new_web['url'] = config.get_value(self._section, '{}{}'.format(postfix, 'url'))
                 if new_web['url'] is None:
                     continue
                 url_parse = urlparse(new_web['url'])
                 if not bool(url_parse.scheme) or not bool(url_parse.netloc):
                     logger.error(
-                        'Bad URL (%s) in the [%s] section of configuration file.' % (new_web['url'], self._section)
+                        'Bad URL ({}) in the [{}] section of configuration file.'.format(new_web['url'], self._section)
                     )
                     continue
 
                 # Read optionals configuration keys
                 # Default description is the URL without the http://
                 new_web['description'] = config.get_value(
-                    self._section, '%sdescription' % postfix, default="%s" % url_parse.netloc
+                    self._section, f'{postfix}description', default=f"{url_parse.netloc}"
                 )
 
                 # Default status
@@ -73,10 +72,10 @@ class GlancesWebList(object):
                 new_web['refresh'] = refresh
 
                 # Timeout in second
-                new_web['timeout'] = int(config.get_value(self._section, '%stimeout' % postfix, default=timeout))
+                new_web['timeout'] = int(config.get_value(self._section, f'{postfix}timeout', default=timeout))
 
                 # RTT warning
-                new_web['rtt_warning'] = config.get_value(self._section, '%srtt_warning' % postfix, default=None)
+                new_web['rtt_warning'] = config.get_value(self._section, f'{postfix}rtt_warning', default=None)
                 if new_web['rtt_warning'] is not None:
                     # Convert to second
                     new_web['rtt_warning'] = int(new_web['rtt_warning']) / 1000.0
@@ -85,11 +84,11 @@ class GlancesWebList(object):
                 new_web['indice'] = 'web_' + str(i)
 
                 # ssl_verify
-                new_web['ssl_verify'] = config.get_value(self._section, '%sssl_verify' % postfix, default=True)
+                new_web['ssl_verify'] = config.get_value(self._section, f'{postfix}ssl_verify', default=True)
                 # Proxy
-                http_proxy = config.get_value(self._section, '%shttp_proxy' % postfix, default=None)
+                http_proxy = config.get_value(self._section, f'{postfix}http_proxy', default=None)
 
-                https_proxy = config.get_value(self._section, '%shttps_proxy' % postfix, default=None)
+                https_proxy = config.get_value(self._section, f'{postfix}https_proxy', default=None)
 
                 if https_proxy is None and http_proxy is None:
                     new_web['proxies'] = None
@@ -97,11 +96,11 @@ class GlancesWebList(object):
                     new_web['proxies'] = {'http': http_proxy, 'https': https_proxy}
 
                 # Add the server to the list
-                logger.debug("Add Web URL %s to the static list" % new_web['url'])
+                logger.debug("Add Web URL {} to the static list".format(new_web['url']))
                 web_list.append(new_web)
 
             # Ports list loaded
-            logger.debug("Web list loaded: %s" % web_list)
+            logger.debug(f"Web list loaded: {web_list}")
 
         return web_list
 
